# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging
import os

from urllib import unquote

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Player import Player

c = logging.getLogger(__name__)

class RhythmboxOld(Player):
    """
        Rhythmbox player plugin
    """
    __title__ = 'RhythmboxOld'
    __version__ = '0.1'
    __authors__ = 'BadChoice <guitarboy000@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Rythmbox music player plugin for versions before 3.0')

    capabilities = ('rating', 'time')

    DBUS_NAME   = 'org.gnome.Rhythmbox'
    DBUS_OBJECT = '/org/gnome/Rhythmbox/Player'
    _player = None
    _shell  = None
    time = 0

    _signals = []

    def _reset(self):
        self._player = None
        self._shell = None
        self._signals = []
        self.connected = False

    def connect(self):
        """
            Connects player
            @return: boolean, True if successful, False if not
        """
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    '/org/gnome/Rhythmbox/Shell')
                self._shell = dbus.Interface(proxy, self.DBUS_NAME+".Shell")

                proxy = dbus.SessionBus().get_object(self.DBUS_NAME,
                    '/org/gnome/Rhythmbox/Player')
                self._player = dbus.Interface(proxy, self.DBUS_NAME+".Player")

                self._signals.append(self._player.connect_to_signal(
                    'playingChanged', self.playing_changed_handler))
                self._signals.append(self._player.connect_to_signal(
                    'playingUriChanged', self.song_changed_handler))
                self._signals.append(self._player.connect_to_signal(
                    'elapsedChanged', self.time_changed_handler))

                self.connected = True

            if self.get_state() in ('playing', 'paused'):
                self.time = self._player.getElapsed()
            else:
                self.time = 0
        except:
            self.disconnect()

        return self.connected

    def disconnect(self):
        """
            Disconnects player
        """
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._shell = None
        self._signals = []
        self.connected = False

    def get_cover_path(self, data):
        """
            @param data: dictionary, contains info about current playing song.
                optional
            @return: string, path to the album art image of current playing
                songm, (not URI)
        """
        try:
            filename = os.path.expanduser('~/.cache/rhythmbox/covers/{0} - {1}.jpg'.format(data['artist'],data['album']))
            if os.path.exists(filename):
                return filename
        except:
            pass
        return ''

    def get_state(self):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        if self._player.getPlaying():
            return State.PLAYING
        else:
            if self._player.getPlayingUri() == '':
                return State.STOPPED
            else:
                return State.PAUSED

    def previous(self):
        """
            Changes song to previous
        """
        if self.connected:
            self._player.previous()

    def play(self):
        """
            Plays or pauses the player
        """
        if self.connected:
            self._player.playPause(True)


    def next(self):
        """
            Changes song to next
        """
        if self.connected:
            self._player.next()

    def getProperty(self, name):
        try:
            val = self._shell.getSongProperties(self._player.getPlayingUri())[name]
            return val
        except:
            return None

    def get_track(self, data=None):
        """
            Prepares a Track object with current playing song data

            @param data: dictionary, contains info about current playing song.
                optional
            @return: Track, object with current playing song data
        """
        if not data:
            try:
                data = self._shell.getSongProperties(self._player.getPlayingUri())
            except:
                return Track()

        vals = ('title','album','artist', 'genre', 'year', 'track-number',
                'location', 'genre')
        for key in [key for key in vals if key not in data]:
            c.warning('%s: %s not found in current track' % ('RHYTHMBOX', key))
            data[key] = ''

        track = Track(
            title = data['title'],
            album = data['album'],
            artist = data['artist'],
            rating = data['rating'],
            genre = data['genre'],
            year = data['year'],
            tracknumber = data['track-number'],
            location = unquote(data['location'].encode('utf-8')),
            cover = self.get_cover_path(data),
            length = data['duration'])
        return track

    def set_rating(self,rating):
        if 0 <= rating and rating <= 5:
            self._shell.setSongProperty(self._player.getPlayingUri(),
                "rating", dbus.Double(float(rating)))

    def get_position(self):
        return self.time

    def set_position(self, time):
        self._player.setElapsed(time)

# ================================= HANDLERS ================================= #

    def playing_changed_handler(self, message):
        if message:
            self.cg.update(STATE, State.PLAYING)
        else:
            self.cg.update(STATE, State.PAUSED)

    def song_changed_handler(self, message):
        self.cg.update(TRACK, self.get_track())

    def time_changed_handler(self, time):
        self.time = int(time)
